<?php

namespace Assetise\Scheduler\Http\Controllers;

use Exception;
use Assetise\Scheduler\Actions\SchedulerWorkflow;
use Assetise\Scheduler\Http\Requests\SchedulerWorkflowRequest;
use Assetise\Scheduler\Http\Resources\SchedulerResource;
use Assetise\Scheduler\Models\Scheduler;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class SchedulerWorkflowController extends BaseController
{
    /**
     * Action controller function for scheduler.
     *
     * @param Model $scheduler
     * @param action next action for the scheduler.
     *
     * @return Response
     */
    public function __invoke(SchedulerWorkflowRequest $request, Scheduler $scheduler, $trasition)
    {
        try {
            $request = $request->all();
            $scheduler = SchedulerWorkflow::run($trasition, $scheduler, $request);
            $data = new SchedulerResource($scheduler);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('scheduler::scheduler.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('scheduler/scheduler/' . $scheduler->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('scheduler/scheduler/' . $scheduler->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the scheduler and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $scheduler
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $scheduler, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $scheduler = Scheduler::findBySignedId($scheduler);
        $transitions = $this->transitions($scheduler, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('scheduler::scheduler.name') . ' ' . trans('Approvals'))
            ->view('scheduler::public.scheduler.workflow')
            ->layout('mobile')
            ->data(compact('scheduler', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the scheduler.
     *
     * @param PublicRequest $request
     * @param string $scheduler
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $scheduler, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $scheduler = Scheduler::findBySignedId($scheduler);
        $user = user();
        $scheduler = SchedulerWorkflow::run($transition, $scheduler, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($scheduler, $user)
    {
        $transitions = [];
        foreach ($scheduler->workflow()->transitions($scheduler) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.scheduler.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'scheduler' => $scheduler->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $scheduler->workflow()->form($value);
            $array['label'] = trans('scheduler::scheduler.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
